import { Book, Copy, HoldRequest, Repository } from '@/payload-types'
import { getPayload, PaginatedDocs } from 'payload'
import configPromise from '@payload-config'
import { headers as getHeaders } from 'next/headers.js'
import BookByIdPageClient from './page.client'
import { PageBreadCrumb, Route } from '@/components/PageBreadCrumb'

const staticBreadcrumRoutes: Route[] = [
  {
    label: 'Home',
    href: '/',
  },
  {
    label: 'Search',
    href: '/search',
  },
]

type Params = Promise<{ bookId: string }>
type SearchParams = Promise<{ [key: string]: string | undefined }>

type Props = {
  params: Params
  searchParams: SearchParams
}
const BookByIdPage = async (props: Props) => {
  const params = await props.params

  const bookIdFromUrl = parseInt(params?.bookId || '', 10)
  if (Number.isNaN(bookIdFromUrl)) return null
  const bookId = bookIdFromUrl as number

  const payload = await getPayload({ config: configPromise })

  const headers = await getHeaders()
  const { user } = await payload.auth({ headers })

  const foundBook = (await payload.findByID({
    collection: 'books',
    depth: 3,
    overrideAccess: false,
    id: bookId,
    select: {
      title: true,
      authors: true,
      publication: true,
      lcc: true,
      genre: true,
      isbn: true,
      copies: true,
      summary: true,
      description: true,
    },
  })) as Book

  let existingHolds: PaginatedDocs<HoldRequest> | null = null
  if (user?.id)
    existingHolds = (await payload.find({
      collection: 'holdRequests',
      depth: 1,
      limit: 1,
      select: {
        dateRequested: true,
        holdingUntilDate: true,
      },
      where: {
        and: [
          {
            'book.id': {
              equals: bookId,
            },
          },
          {
            userRequested: {
              equals: user.id,
            },
          },
          {
            isCheckedOut: {
              not_equals: true,
            },
          },
          {
            isRejected: {
              not_equals: true,
            },
          },
        ],
      },
    })) as PaginatedDocs<HoldRequest>

  const defaultRepositoryLimit = 10
  const repositoryIds = foundBook.copies?.docs?.map((c) => (c as Copy).repository)
  const repositoryOrQueries = repositoryIds?.map((c) => ({ id: { equals: c } }))
  const repositories = (await payload.find({
    collection: 'repositories',
    depth: 3,
    limit: defaultRepositoryLimit,
    select: {
      id: true,
      owners: true,
      abbreviation: true,
      name: true,
    },
    where: {
      or: repositoryOrQueries?.length ? repositoryOrQueries : [],
    },
  })) as PaginatedDocs<Repository>

  const createBreadcrumbRoutes = () => {
    const maxTitleLength = 18

    let title
    if (!foundBook?.title) title = 'Book Not Found'
    else
      title =
        foundBook.title.length > maxTitleLength
          ? `${foundBook.title.slice(0, maxTitleLength)}...`
          : foundBook.title

    return [
      ...staticBreadcrumRoutes,
      {
        label: title,
        href: `/books/${bookId}`,
      },
    ]
  }

  return (
    <>
      <PageBreadCrumb routes={createBreadcrumbRoutes()} />
      <BookByIdPageClient
        book={foundBook}
        repositories={repositories}
        existingHolds={existingHolds}
      />
    </>
  )
}

export default BookByIdPage
