'use client'

import {
  Dialog,
  DialogDescription,
  DialogContent,
  DialogHeader,
  DialogTitle,
  DialogClose,
} from '@/components/ui/dialog'
import { Book, Copy, HoldRequest, Repository, User } from '@/payload-types'
import { findCopiesOfBookInRepository } from '@/serverActions/FindCopiesOnHoldRequest'
import { Variants, Transition } from 'motion/react'
import { useEffect, useState } from 'react'
import { Input } from '../ui/input'
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form'
import { useForm } from 'react-hook-form'
import { z } from 'zod'
import { zodResolver } from '@hookform/resolvers/zod'
import { Button } from '../ui/button'
import { Select } from '@headlessui/react'
import approveHoldRequest from '@/serverActions/ApproveHoldRequests'
import { toast } from 'sonner'

const formSchema = z
  .object({
    copyId: z.string(),
    untilDate: z.string().date(),
  })
  .required()

type Props = {
  holdRequest: HoldRequest
  isOpen: boolean
  onOpenChange: () => void
}
const ApproveHoldRequestModal = (props: Props) => {
  const { holdRequest, isOpen, onOpenChange } = props

  const customVariants: Variants = {
    initial: {
      scale: 0.9,
      filter: 'blur(10px)',
      y: '100%',
    },
    animate: {
      scale: 1,
      filter: 'blur(0px)',
      y: 0,
    },
  }

  const customTransition: Transition = {
    type: 'spring',
    bounce: 0,
    duration: 0.4,
  }

  const hold = holdRequest as HoldRequest
  const book = hold.book as Book
  const repository = hold.repository as Repository
  const userRequested = hold.userRequested as User

  const [availableCopies, setAvailableCopies] = useState<Copy[]>([])

  const form = useForm<z.infer<typeof formSchema>>({
    resolver: zodResolver(formSchema),
    defaultValues: {
      copyId: undefined,
      untilDate: undefined,
    },
  })

  useEffect(() => {
    if (!isOpen || availableCopies.length) return

    findCopiesOfBookInRepository({
      bookId: book.id,
      repositoryId: repository.id,
    })
      .then(async (response) => {
        const copies = response?.docs || []
        const availableCopies = copies.filter((c) => !c.holdRequests?.docs?.length)
        console.log(availableCopies)
        setAvailableCopies(availableCopies)
      })
      .catch((err) => {
        console.error(err)
      })
      .finally(() => {})
  }, [availableCopies, setAvailableCopies, isOpen, book.id, repository.id])

  const onSubmit = async (values: z.infer<typeof formSchema>) => {
    const updateRequest = await approveHoldRequest({
      holdRequestId: hold.id,
      copyId: parseInt(values.copyId, 10),
      untilDate: values.untilDate,
    })

    if (!updateRequest) toast('There was an issue approving that request.')
    else toast('Approved')

    onOpenChange()
  }

  return (
    <Dialog
      open={isOpen}
      onOpenChange={onOpenChange}
      variants={customVariants}
      transition={customTransition}
    >
      <DialogContent className="w-full max-w-md bg-white p-6 dark:bg-zinc-900">
        <DialogHeader>
          <DialogTitle className="text-zinc-900 dark:text-white">
            Hold a copy for {`${userRequested.firstName} ${userRequested.lastName}`}?
          </DialogTitle>
          <DialogDescription className="text-zinc-600 dark:text-zinc-400">
            Select which physical copy, and until when, you would like to hold.
          </DialogDescription>
        </DialogHeader>
        <Form {...form}>
          <form onSubmit={form.handleSubmit(onSubmit)} className="mt-6 flex flex-col space-y-4">
            <FormField
              control={form.control}
              name="copyId"
              render={({ field }) => (
                <FormItem className="overflow-hidden">
                  <FormLabel>Copy</FormLabel>
                  <FormControl>
                    <Select {...field} className="w-full">
                      <option>Please Select a Copy</option>
                      {availableCopies.map((c) => {
                        console.log(c)
                        return (
                          <option key={c.id + '_copy'} value={c.id}>
                            {c.label || `[${repository.abbreviation}] ${book.title}`}
                          </option>
                        )
                      })}
                    </Select>
                  </FormControl>
                  <FormMessage />
                </FormItem>
              )}
            />

            <FormField
              control={form.control}
              name="untilDate"
              render={({ field }) => (
                <FormItem className="sm:w-auto w-full block">
                  <FormLabel>Until</FormLabel>
                  <FormControl>
                    <Input type="date" {...field} />
                  </FormControl>
                  <FormMessage />
                </FormItem>
              )}
            />

            <Button
              className="mt-4 cursor-pointer inline-flex items-center justify-center self-end rounded-lg bg-black px-4 py-2 text-sm font-medium text-zinc-50 dark:bg-white dark:text-zinc-900"
              type="submit"
            >
              Approve Hold
            </Button>
          </form>
        </Form>
        <DialogClose />
      </DialogContent>
    </Dialog>
  )
}

export default ApproveHoldRequestModal
