'use client'

import { Avatar } from '@/components/avatar'
import {
  Dropdown,
  DropdownButton,
  DropdownDivider,
  DropdownItem,
  DropdownLabel,
  DropdownMenu,
} from '@/components/dropdown'
import {
  Navbar,
  NavbarDivider,
  NavbarItem,
  NavbarLabel,
  NavbarSection,
  NavbarSpacer,
} from '@/components/navbar'
import {
  Sidebar,
  SidebarBody,
  SidebarHeader,
  SidebarItem,
  SidebarLabel,
  SidebarSection,
} from '@/components/sidebar'
import { StackedLayout } from '@/components/stacked-layout'
import { Media } from '@/payload-types'
import { useGlobal } from '@/providers/GlobalProvider'
import { ArrowRightStartOnRectangleIcon, UserIcon } from '@heroicons/react/16/solid'
import { MagnifyingGlassIcon, SunIcon, MoonIcon } from '@heroicons/react/20/solid'
import { useTheme } from 'next-themes'
import Link from 'next/link'
import React, { ReactNode, useMemo } from 'react'

type NavItem = {
  label: string
  href: string
  newTab: boolean
}

type Props = {
  children: ReactNode
  navItems: NavItem[]
}
export default function SiteNavigation(props: Props) {
  const { children, navItems } = props
  const { theme, setTheme } = useTheme()
  const { user, setUser } = useGlobal()

  useMemo(() => {
    if (user) return
    fetch('/api/users/me', {
      method: 'GET',
      credentials: 'include',
      headers: {
        'Content-Type': 'application/json',
      },
    }).then(async (response) => {
      const userRequest = await response.json()
      setUser(userRequest.user)
    })
  }, [user, setUser])

  const profilePicture = user?.profilePicture as Media | undefined
  const initials = user?.firstName
    ? `${user?.firstName.slice(0, 1) || ''}${user?.lastName?.slice(0, 1)}`
    : 'U'

  return (
    <StackedLayout
      navbar={
        <Navbar>
          <Dropdown>
            <DropdownButton as={NavbarItem} className="max-lg:hidden">
              <Link href="/" className="flex items-center justify-around gap-3">
                <Avatar src="/api/media/file/bethel-logo.jpg" className="size-8" />
                <NavbarLabel>Midrashim</NavbarLabel>
              </Link>
            </DropdownButton>
          </Dropdown>
          <NavbarDivider className="max-lg:hidden" />
          <NavbarSection className="max-lg:hidden">
            {navItems.map(({ label, href, newTab }) => (
              <NavbarItem key={label} target={newTab ? '_blank' : ''} href={href}>
                {label}
              </NavbarItem>
            ))}
          </NavbarSection>
          <NavbarSpacer />
          <NavbarSection>
            <NavbarItem href="/search" aria-label="Search">
              <MagnifyingGlassIcon />
            </NavbarItem>
            <NavbarItem
              aria-label="Inbox"
              role={'button'}
              onClick={() => setTheme(theme === 'dark' ? 'light' : 'dark')}
            >
              <SunIcon className="dark:hidden" />
              <MoonIcon className="hidden dark:block" />
            </NavbarItem>
            <Dropdown>
              <DropdownButton as={NavbarItem}>
                <Avatar src={profilePicture?.url || ''} initials={initials} square={false} />
              </DropdownButton>
              <DropdownMenu className="min-w-64" anchor="bottom end">
                {!user ? (
                  <DropdownItem href="/login">
                    <UserIcon />
                    <DropdownLabel>Login</DropdownLabel>
                  </DropdownItem>
                ) : (
                  <DropdownItem href="/profile">
                    <UserIcon />
                    <DropdownLabel>My profile</DropdownLabel>
                  </DropdownItem>
                )}

                {!!user && (
                  <>
                    <DropdownDivider />
                    <DropdownItem href="/logout">
                      <ArrowRightStartOnRectangleIcon />
                      <DropdownLabel>Sign out</DropdownLabel>
                    </DropdownItem>
                  </>
                )}
              </DropdownMenu>
            </Dropdown>
          </NavbarSection>
        </Navbar>
      }
      sidebar={
        <Sidebar>
          <SidebarHeader>
            <Dropdown>
              <DropdownButton as={SidebarItem} className="lg:mb-2.5">
                <Link href="/" className="flex items-center justify-around gap-3">
                  <Avatar src="/api/media/file/bethel-logo.jpg" className="size-8" />
                  <SidebarLabel>Midrashim</SidebarLabel>
                </Link>
              </DropdownButton>
            </Dropdown>
          </SidebarHeader>
          <SidebarBody>
            <SidebarSection>
              {navItems.map(({ label, href, newTab }) => (
                <SidebarItem key={label} href={href} target={newTab ? '_blank' : ''}>
                  {label}
                </SidebarItem>
              ))}
            </SidebarSection>
          </SidebarBody>
        </Sidebar>
      }
    >
      {children}
    </StackedLayout>
  )
}
